@@ -22,8 +22,8 @@
 #define INDEXEDDATARETRIEVER_H
 
 #include <KJob>
-#include <QtCore/QProcess>
-#include <QtCore/QVariant>
+#include <QProcess>
+#include <QVariant>
 
 namespace Baloo {
 
@@ -31,20 +31,22 @@ class IndexedDataRetriever : public KJob
 {
     Q_OBJECT
 public:
-    IndexedDataRetriever(const QString& fileUrl, QObject* parent = 0);
-    virtual ~IndexedDataRetriever();
+    explicit IndexedDataRetriever(const QString& fileUrl, QObject* parent = nullptr);
+    ~IndexedDataRetriever() override;
 
-    virtual void start() Q_DECL_OVERRIDE;
+    void start() override;
 
     QVariantMap data() const;
+    bool canEdit() const;
 
 private Q_SLOTS:
-    void slotIndexedFile(int error);
+    void slotIndexedFile(int exitCode, QProcess::ExitStatus exitStatus);
 
 private:
     QString m_url;
     QProcess* m_process;
     QVariantMap m_data;
+    bool m_canEdit = false;
 };
 
 }
