@@ -26,16 +26,18 @@
 #include "KRatingWidget"
 
 #include <KFileMetaData/UserMetaData>
+#include <KFileMetaData/PropertyInfo>
 
-#include <QtWidgets/QLabel>
-#include <QtCore/QCollator>
-#include <QtCore/QTime>
-#include <QtCore/QUrl>
-#include <QtCore/QLocale>
+#include <QLabel>
+#include <QCollator>
+#include <QTime>
+#include <QUrl>
+#include <QLocale>
 
 #include <KJob>
 #include <KFormat>
 #include <KLocalizedString>
+#include <KStringHandler>
 
 namespace {
     static QString plainText(const QString& richText)
@@ -65,6 +67,7 @@ WidgetFactory::WidgetFactory(QObject* parent)
     : QObject(parent)
     , m_readOnly( false )
     , m_noLinks( false )
+    , m_dateFormat(QLocale::LongFormat)
 {
 }
 
@@ -75,21 +78,44 @@ WidgetFactory::~WidgetFactory()
 //
 // Widget Creation
 //
-static QString toString(const QVariant& value)
+static QString formatDateTime(const QVariant& value, QLocale::FormatType dateFormat)
+{
+    const QString valueString = value.toString();
+    QDateTime dt = QDateTime::fromString(valueString, Qt::ISODate);
+
+    if (dt.isValid()) {
+        KFormat form;
+        QTime time = dt.time();
+        // Check if Date/DateTime
+        if (!time.hour() && !time.minute() && !time.second()){
+            return form.formatRelativeDate(dt.date(), dateFormat);
+        } else {
+            return form.formatRelativeDateTime(dt, dateFormat);
+        }
+    }
+
+    return valueString;
+}
+
+static QString toString(const QVariant& value, QLocale::FormatType dateFormat)
 {
     switch (value.type()) {
         case QVariant::Int:
             return QLocale().toString(value.toInt());
         case QVariant::Double:
             return QLocale().toString(value.toDouble());
-
+        case QVariant::StringList:
+            return value.toStringList().join(i18nc("String list separator", ", "));
+        case QVariant::Date:
+        case QVariant::DateTime: {
+            return formatDateTime(value, dateFormat);
+        }
         case QVariant::List: {
             QStringList list;
             for (const QVariant& var : value.toList()) {
-                list << toString(var);
+                list << toString(var, dateFormat);
             }
-
-            return list.join(", ");
+            return list.join(i18nc("String list separator", ", "));
         }
 
         default:
@@ -99,7 +125,8 @@ static QString toString(const QVariant& value)
 
 QWidget* WidgetFactory::createWidget(const QString& prop, const QVariant& value, QWidget* parent)
 {
-    QWidget* widget = 0;
+    QWidget* widget = nullptr;
+    const int maxUrlLength = 80;
 
     if (prop == QLatin1String("rating")) {
         widget = createRatingWidget( value.toInt(), parent );
@@ -115,42 +142,36 @@ QWidget* WidgetFactory::createWidget(const QString& prop, const QVariant& value,
         widget = createTagWidget( tags, parent );
     }
     else {
-        KFormat form;
-        // vHanda: FIXME: Add links! Take m_noLinks into consideration
         QString valueString;
-
-        if (prop == QStringLiteral("duration")) {
-            valueString = form.formatDuration(value.toInt() * 1000);
-        } else if (prop == QStringLiteral("bitRate")) {
-            valueString = i18nc("@label bitrate (per second)", "%1/s", form.formatByteSize(value.toInt(), 1, KFormat::MetricBinaryDialect));
-        } else if (prop == QLatin1String("originUrl")) {
-            if (m_noLinks) {
-                valueString = value.toString();
+        auto pi = KFileMetaData::PropertyInfo::fromName(prop);
+        if (pi.name() == QLatin1String("originUrl")) {
+            //Won't make sense to shrink originUrl with noLinks,
+            //since it would make original URL unobtainable
+            valueString = value.toString();
+            if (!m_noLinks) {
+                //Shrink link name.
+                auto labelString = valueString;
+                if (labelString.size() > maxUrlLength) {
+                    labelString = KStringHandler::csqueeze(labelString, maxUrlLength);
+                }
+                valueString = QStringLiteral("<a href=\"%1\">%2</a>").arg(valueString, labelString);
+            }
+        } else if (pi.name() != QLatin1String("empty")) {
+            if (pi.valueType() == QVariant::DateTime || pi.valueType() == QVariant::Date) {
+                valueString = formatDateTime(value, m_dateFormat);
             } else {
-                valueString = QStringLiteral("<a href=\"%1\">%1</a>").arg(value.toString());
+                valueString = pi.formatAsDisplayString(value);
             }
         } else {
-            // Check if Date/DateTime
-
-            QDateTime dt = QDateTime::fromString(value.toString(), Qt::ISODate);
-            if (dt.isValid()) {
-                QTime time = dt.time();
-                if (!time.hour() && !time.minute() && !time.second()){
-                    valueString = form.formatRelativeDate(dt.date(), QLocale::LongFormat);
-                } else {
-                    valueString = form.formatRelativeDateTime(dt, QLocale::LongFormat);
-                }
-            }
-            else {
-                valueString = toString(value);
-            }
+            valueString = toString(value, m_dateFormat);
         }
+
         widget = createValueWidget(valueString, parent);
     }
 
     widget->setForegroundRole(parent->foregroundRole());
     widget->setFont(parent->font());
-
+    widget->setObjectName(prop);
     return widget;
 }
 
@@ -209,8 +230,8 @@ QWidget* WidgetFactory::createRatingWidget(int rating, QWidget* parent)
 class ValueWidget : public QLabel
 {
 public:
-    explicit ValueWidget(QWidget* parent = 0);
-    virtual QSize sizeHint() const Q_DECL_OVERRIDE;
+    explicit ValueWidget(QWidget* parent = nullptr);
+    QSize sizeHint() const override;
 };
 
 ValueWidget::ValueWidget(QWidget* parent) :
@@ -231,6 +252,7 @@ QSize ValueWidget::sizeHint() const
 QWidget* WidgetFactory::createValueWidget(const QString& value, QWidget* parent)
 {
     ValueWidget* valueWidget = new ValueWidget(parent);
+    valueWidget->setTextInteractionFlags(Qt::TextSelectableByMouse | Qt::TextSelectableByKeyboard);
     valueWidget->setWordWrap(true);
     valueWidget->setAlignment(Qt::AlignTop | Qt::AlignLeft);
     valueWidget->setText(m_readOnly ? plainText(value) : value);
@@ -245,8 +267,12 @@ QWidget* WidgetFactory::createValueWidget(const QString& value, QWidget* parent)
 
 void WidgetFactory::slotCommentChanged(const QString& comment)
 {
-    for (const QString& filePath : m_items) {
-        KFileMetaData::UserMetaData md(filePath);
+    for (const KFileItem& item : qAsConst(m_items)) {
+        QUrl url = item.targetUrl();
+        if (!url.isLocalFile()) {
+            continue;
+        }
+        KFileMetaData::UserMetaData md(url.toLocalFile());
         md.setUserComment(comment);
     }
     emit dataChangeStarted();
@@ -255,8 +281,12 @@ void WidgetFactory::slotCommentChanged(const QString& comment)
 
 void WidgetFactory::slotRatingChanged(uint rating)
 {
-    for (const QString& filePath : m_items) {
-        KFileMetaData::UserMetaData md(filePath);
+    for (const KFileItem& item : qAsConst(m_items)) {
+        QUrl url = item.targetUrl();
+        if (!url.isLocalFile()) {
+            continue;
+        }
+        KFileMetaData::UserMetaData md(url.toLocalFile());
         md.setRating(rating);
     }
     emit dataChangeStarted();
@@ -266,8 +296,12 @@ void WidgetFactory::slotRatingChanged(uint rating)
 void WidgetFactory::slotTagsChanged(const QStringList& tags)
 {
     if (m_tagWidget) {
-        for (const QString& filePath : m_items) {
-            KFileMetaData::UserMetaData md(filePath);
+        for (const KFileItem& item : qAsConst(m_items)) {
+            QUrl url = item.targetUrl();
+            if (!url.isLocalFile()) {
+                continue;
+            }
+            KFileMetaData::UserMetaData md(url.toLocalFile());
 
             // When multiple tags are selected one doesn't want to loose the old tags
             // of any of the resources. Unless specifically removed.
@@ -300,7 +334,7 @@ void WidgetFactory::slotLinkActivated(const QString& url)
 void WidgetFactory::slotTagClicked(const QString& tag)
 {
     QUrl url;
-    url.setScheme("tags");
+    url.setScheme(QStringLiteral("tags"));
     url.setPath(tag);
 
     emit urlActivated(url);
@@ -320,8 +354,18 @@ void WidgetFactory::setNoLinks(bool value)
     m_noLinks = value;
 }
 
-void WidgetFactory::setItems(const QStringList& items)
+void WidgetFactory::setItems(const KFileItemList& items)
 {
     m_items = items;
 }
 
+Baloo::DateFormats WidgetFactory::dateFormat() const
+{
+    return static_cast<Baloo::DateFormats>(m_dateFormat);
+}
+
+void Baloo::WidgetFactory::setDateFormat(const Baloo::DateFormats format)
+{
+    m_dateFormat = static_cast<QLocale::FormatType>(format);
+}
+
