@@ -21,11 +21,13 @@
 #ifndef WIDGETFACTORY_H
 #define WIDGETFACTORY_H
 
+#include "filemetadatawidget.h"
+
 #include <QObject>
 #include <QStringList>
 
 class KJob;
-class KUrl;
+class QUrl;
 class KCommentWidget;
 class KRatingWidget;
 
@@ -38,22 +40,25 @@ namespace Baloo {
     {
         Q_OBJECT
     public:
-        explicit WidgetFactory(QObject* parent = 0);
-        virtual ~WidgetFactory();
+        explicit WidgetFactory(QObject* parent = nullptr);
+        ~WidgetFactory() override;
 
-        void setItems(const QStringList& items);
+        void setItems(const KFileItemList& items);
 
         void setReadOnly(bool value);
         void setNoLinks(bool value);
 
+        void setDateFormat(const DateFormats format);
+        DateFormats dateFormat() const;
+
         QWidget* createWidget(const QString& prop, const QVariant& value, QWidget* parent);
 
-    signals:
-        void urlActivated(const KUrl& url);
+    Q_SIGNALS:
+        void urlActivated(const QUrl& url);
         void dataChangeStarted();
         void dataChangeFinished();
 
-    private slots:
+    private Q_SLOTS:
         void slotTagsChanged(const QStringList& tags);
         void slotCommentChanged(const QString& comment);
         void slotRatingChanged(uint rating);
@@ -67,16 +72,15 @@ namespace Baloo {
         QWidget* createCommentWidget(const QString& comment, QWidget* parent);
         QWidget* createValueWidget(const QString& value, QWidget* parent);
 
-        void startChangeDataJob(KJob* job);
-
         TagWidget* m_tagWidget;
         KRatingWidget* m_ratingWidget;
         KCommentWidget* m_commentWidget;
 
-        QStringList m_items;
+        KFileItemList m_items;
         QStringList m_prevTags;
         bool m_readOnly;
         bool m_noLinks;
+        QLocale::FormatType m_dateFormat;
     };
 }
 
