@@ -20,25 +20,23 @@
 #include "filemetadataconfigwidget.h"
 
 #include <QApplication>
-#include <kcomponentdata.h>
-#include <kfiledialog.h>
-#include <KPushButton>
-
+#include <QPushButton>
 #include <QVBoxLayout>
 #include <QCheckBox>
+#include <QFileDialog>
 
 class FileMetadataWidgetTest : public QWidget
 {
     Q_OBJECT
 public:
-    explicit FileMetadataWidgetTest(QWidget* parent = 0, Qt::WindowFlags f = 0);
+    explicit FileMetadataWidgetTest(QWidget* parent = nullptr, Qt::WindowFlags f = {});
 
-private slots:
+private Q_SLOTS:
     void slotChooseFiles();
 
 private:
     Baloo::FileMetaDataConfigWidget* m_metadataWidget;
-    KPushButton* m_button;
+    QPushButton* m_button;
 };
 
 FileMetadataWidgetTest::FileMetadataWidgetTest(QWidget* parent, Qt::WindowFlags f)
@@ -46,7 +44,7 @@ FileMetadataWidgetTest::FileMetadataWidgetTest(QWidget* parent, Qt::WindowFlags
 {
     m_metadataWidget = new Baloo::FileMetaDataConfigWidget( this );
 
-    m_button = new KPushButton( QLatin1String("Select files"), this );
+    m_button = new QPushButton( QLatin1String("Select files"), this );
     connect( m_button, SIGNAL(clicked(bool)), this, SLOT(slotChooseFiles()) );
 
     QVBoxLayout* layout = new QVBoxLayout( this );
@@ -56,9 +54,9 @@ FileMetadataWidgetTest::FileMetadataWidgetTest(QWidget* parent, Qt::WindowFlags
 
 void FileMetadataWidgetTest::slotChooseFiles()
 {
-    KUrl::List urlList = KFileDialog::getOpenUrls();
+    const QList<QUrl> urlList = QFileDialog::getOpenFileUrls();
     KFileItemList list;
-    foreach(const KUrl& url, urlList)
+    for (const QUrl& url : urlList)
         list << KFileItem( url, QString(), mode_t() );
 
     m_metadataWidget->setItems( list );
@@ -67,7 +65,7 @@ void FileMetadataWidgetTest::slotChooseFiles()
 int main( int argc, char** argv )
 {
     QApplication app( argc, argv );
-    KComponentData data( "FileMetaDataConfigWidgetApp" );
+    app.setApplicationName(QStringLiteral("FileMetaDataConfigWidgetApp"));
     FileMetadataWidgetTest test;
     test.show();
     return app.exec();
